package com.framsticks.core;

import com.framsticks.util.lang.IterableIterator;
import com.framsticks.util.lang.Pair;
import com.framsticks.util.lang.Strings;
import org.apache.commons.configuration.*;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import java.lang.reflect.Constructor;
import java.util.*;

/**
 * @author Piotr Sniegowski
 */
public class Program extends com.framsticks.util.dispatching.Thread {

	private final static Logger log = Logger.getLogger(Program.class.getName());

	protected Entity entity;

	Configuration config;

	public Program(String name) {
		super(name, java.lang.Thread.currentThread());
	}

	public static Entity configureEntity(Configuration config) {
		String typeName = config.getString("class");
		log.info("configuring instance " + typeName);
		try {
			Class<?> type = Class.forName(typeName);
			Constructor<?> constructor = type.getConstructor();
			Entity entity = (Entity) constructor.newInstance();
			return entity;
		} catch (Exception e) {
			log.error("failed to instantiate: " + e);
		}
		return null;
	}

	protected static void resolve(Configuration config, String prefix) {
		for (String p : new IterableIterator<String>(prefix == null ? config.getKeys() : config.getKeys(prefix))) {
			if (p.endsWith(".mount")) {
				String source = config.getString(p);
				log.info("mounting " + source + " at " + p);
				config.clearProperty(p);
				Configuration mount = config.subset(source);
				for (String mk : new IterableIterator<String>(mount.getKeys())) {
					config.addProperty(p.substring(0, p.length() - 6) + "."	+ mk, mount.getProperty(mk));
				}
			}
		}
	}

	public void run(String[] args) {

		PropertyConfigurator.configure(getClass().getResource("/configs/log4j.properties"));
		log.debug("started in " + System.getProperty("user.dir"));
		try {
			config = new PropertiesConfiguration(getClass().getResource("/configs/framsticks.properties"));

			for (String a : args) {
				Pair<String, String> p = Strings.splitIntoPair(a, '=', "true");
				config.setProperty(p.first, p.second);
			}

			resolve(config, null);

			Iterator<?> i = config.getKeys();
			while (i.hasNext()) {
				String p = (String) i.next();
				log.debug(p + " = " + config.getProperty(p));
			}

		} catch (ConfigurationException e) {
			System.err.print("failed to parse configuration:" + e);
			return;
		}
		Configuration entityConfig = config.subset("com.framsticks.entity");

		entity = configureEntity(entityConfig);
		entity.setOwner(new EntityOwner() {
			@Override
			public void onDone() {
				log.info("exiting");
				Runtime.getRuntime().exit(0);
			}
		});
		entity.setName("main");
		try {
			entity.configure(entityConfig);
		} catch (Exception e) {
			log.fatal("exception caught during configuration: " + e);
		}
		log.info("all entities were configured");
		entity.start();
	}

	public static void main(final String[] args) {

		final Program program = new Program("program");
		program.invokeLater(new Runnable() {
			@Override
			public void run() {
				program.run(args);
			}
		});
		program.routine();
	}

}
