package com.framsticks.params;


/**
 * @author Piotr Sniegowski
 */
public abstract class PrimitiveParam extends ValueParam {

	/** The minimum allowed value of parameter. */
	protected Object min;

	/** The maximum allowed value of parameter. */
	protected Object max;

	/** The default value of parameter. */
	protected Object def;

	public void save(SinkInterface sink, Object value) {
		sink.print(value);
	}

	protected <T> T tryCastAndReturn(Object value, Class<T> type) {
		if (value == null)
			return null;
		try {
			return type.cast(value);
		} catch (ClassCastException e) {
			throw new ClassCastException("property \"" + name
					+ "\" type is \"" + value.getClass().getName()
					+ "\", not \"" + type.getName() + "\"");
		}
	}

	/**
	 * Gets the minimum value of parameter.
	 *
	 * @param <T> the generic getType which must be correctly specified by user
	 * @param type the getType of variable, can be checked by
	 * @return the minimum allowed value of parameter
	 * @throws ClassCastException the class cast exception, thrown when given getType is incorrect
	 */
	public <T> T getMin(Class<T> type) {
		return tryCastAndReturn(min, type);
	}

	/**
	 * Gets the maximum value of parameter.
	 *
	 * @param <T> the generic getType which must be correctly specified by user
	 * @param type the getType of variable, can be checked by {@link #getStorageType()}
	 * @return the maximum allowed value of parameter
	 * @throws ClassCastException the class cast exception, thrown when given getType is incorrect
	 */
	public <T> T getMax(Class<T> type) {
		return tryCastAndReturn(max, type);
	}

	/**
	 * Gets the default value of parameter.
	 *
	 * @param <T> the generic getType which must be correctly specified by user
	 * @param type the getType of variable, can be checked by {@link #getStorageType()}
	 * @return the default value of parameter
	 * @throws ClassCastException the class cast exception, thrown when given getType is incorrect
	 */
	public <T> T getDef(Class<T> type) throws ClassCastException {
		return tryCastAndReturn(def, type);
	}
}
